@@ -28,6 +28,7 @@ Decimal = d.Decimal
 import time
 import warnings
 from importlib import import_module
+import copy
 import inspect
 import tempfile
 import numpy as np
@@ -63,7 +64,7 @@ SEARCH = "SEARCH"
 class Archive(object):
     def __init__(self, filename, prepare=True, lowmem=False, verbose=True,
                  weight=True, center_pulse=True, baseline_removal=True,
-                 wcfreq=True, thread=False, cuda=False):
+                 wcfreq=True, thread=False, cuda=False, onlyheader=False):
         ## Parse filename here
         self.pypulse_history = []
         self.record(inspect.currentframe())
@@ -76,14 +77,18 @@ class Archive(object):
         self.wcfreq = wcfreq
         self.thread = thread
         self.cuda = cuda
+        self.onlyheader = onlyheader
 
         if verbose:
-            print("Loading: %s" % self.filename)
+            if onlyheader:
+                print("Loading (header only): %s" % self.filename)
+            else:
+                print("Loading: %s" % self.filename)
             t0 = time.time()
 
         self.load(self.filename, prepare=prepare, center_pulse=center_pulse,
                   baseline_removal=baseline_removal, weight=weight,
-                  wcfreq=wcfreq)
+                  wcfreq=wcfreq, onlyheader=onlyheader)
         if not self.lowmem:
             self.data_orig = np.copy(self.data)
             self.weights_orig = np.copy(self.weights)
@@ -110,7 +115,8 @@ class Archive(object):
         return self.filename
 
     def load(self, filename, prepare=True, center_pulse=True,
-             baseline_removal=True, weight=True, wcfreq=False):
+             baseline_removal=True, weight=True, wcfreq=True,
+             onlyheader=False):
         """
         Loads a PSRFITS file and processes
         http://www.atnf.csiro.au/people/pulsar/index.html?n=PsrfitsDocumentation.Txt
@@ -122,8 +128,8 @@ class Archive(object):
                 hdulist = pyfits.open(filename, ignore_missing_end=True, memmap=True)
             else:
                 hdulist = pyfits.open(filename, ignore_missing_end=True)
-        except IOError:
-            print("Filename not found")
+        except IOError as err:
+            print("IOError: Filename not found")
             raise SystemExit
         self.header = hdulist[0].header
         self.keys = fmap(lambda x: x.name, hdulist)
@@ -139,7 +145,7 @@ class Archive(object):
         else:
             self.history = None
             nsubint = hdulist['SUBINT'].header['NAXIS2']
-            nbin, nchan, npol, nsblk = fmap(int, hdulist['SUBINT'].columns[-1].dim[1:-1].split(","))
+            nbin, nchan, npol, nsblk = fmap(int, hdulist['SUBINT'].columns['DATA'].dim[1:-1].split(","))
 
         if 'PSRPARAM' in self.keys:
             tablenames.remove('PSRPARAM')
@@ -177,7 +183,12 @@ class Archive(object):
             isFluxcal = True
         self.tables = list()
         for tablename in tablenames: #remaining table names to store
-            self.tables.append(hdulist[tablename].copy())
+            try:
+                self.tables.append(hdulist[tablename].copy())
+            except ValueError:
+                # Some other data formats break the above
+                # with unknown tables, so just ignore them
+                continue
 
         #if self.header['OBS_MODE'] == 'PCM':
         if isFluxcal:
@@ -194,6 +205,11 @@ class Archive(object):
         for i, key in enumerate(hdulist['SUBINT'].header):
             self.subintheader[key] = hdulist['SUBINT'].header[key]
 
+        if onlyheader:
+            self.weights = None
+            self._data = None
+            return
+            
         DATA = hdulist['SUBINT'].data['DATA']
         if np.ndim(DATA) == 5:
             DATA = DATA[:, 0, :, :, :] #remove the nsblk column
@@ -299,7 +315,7 @@ class Archive(object):
                     autoinit = import_module('pycuda.autoinit')
                     cudasuccess = True
                 except ImportError:
-                    print("PyCUDA not imported")
+                    warnings.warn("PyCUDA not imported", ImportWarning)
                     #                __global__ void combine(float *retval, float *DAT_SCL, float *DATA, float *DAT_OFFS, int nbin)
                 mod = compiler.SourceModule("""
                 __global__ void combine(int16_t *retval, float *DAT_SCL, int16_t *DATA, float *DAT_OFFS, int nbin, int size)
@@ -578,22 +594,21 @@ class Archive(object):
         retval = np.zeros((len(np.r_[0:nsubint:factor]), npol, nchan, nbin))
         newdurations = np.zeros(np.shape(retval)[0])
 
-        weightsum = np.sum(self.weights)
         weightretval = np.zeros((len(np.r_[0:nsubint:factor]), nchan))
 
         newnsubint = nsubint//factor
         for i in xrange(newnsubint):
-            weightretval[i, :] = np.sum(self.weights[i*factor:(i+1)*factor, :], axis=0)
+            weightretval[i, :] = np.mean(self.weights[i*factor:(i+1)*factor, :], axis=0)
 
         for i in xrange(newnsubint):
             newdurations[i] += np.sum(self.durations[i*factor:(i+1)*factor])
             for j in xrange(npol):
                 for k in xrange(nchan):
                     for l in xrange(nbin):
-                        retval[i, j, k, l] = np.sum(self.data[i*factor:(i+1)*factor, j, k, l] * self.weights[i*factor:(i+1)*factor, k])
+                        retval[i, j, k, l] = np.sum(self.data[i*factor:(i+1)*factor, j, k, l] * self.weights[i*factor:(i+1)*factor, k]) / np.sum(self.weights[i*factor:(i+1)*factor, k])
 
         self.weights = weightretval
-        self.data = retval/weightsum
+        self.data = retval
         self.durations = newdurations
 
         return self
@@ -651,7 +666,6 @@ class Archive(object):
         
         retval = np.zeros((nsubint, npol, len(np.r_[0:nchan:factor]), nbin))
 
-        weightsum = np.sum(self.weights)
         weightretval = np.zeros((nsubint, len(np.r_[0:nchan:factor])))
 
 
@@ -665,7 +679,7 @@ class Archive(object):
         newfreq = np.zeros(newnchan) #this will only be 1D
 
         for k in xrange(newnchan):
-            weightretval[:, k] = np.sum(self.weights[:, k*factor:(k+1)*factor], axis=1)
+            weightretval[:, k] = np.mean(self.weights[:, k*factor:(k+1)*factor], axis=1)
             newfreq[k] = np.sum(freq[k*factor:(k+1)*factor])/float(factor) #unweighted!
 
         for i in xrange(nsubint):
@@ -673,11 +687,11 @@ class Archive(object):
                 for k in xrange(newnchan):
                     for l in xrange(nbin):
                         retval[i, j, k, l] = \
-                        np.sum(self.data[i, j, k*factor:(k+1)*factor, l] * self.weights[i, k*factor:(k+1)*factor])
+                        np.sum(self.data[i, j, k*factor:(k+1)*factor, l] * self.weights[i, k*factor:(k+1)*factor]) / np.sum(self.weights[i, k*factor:(k+1)*factor])
 
 
         self.weights = weightretval
-        self.data = retval/weightsum
+        self.data = retval
         self.freq = newfreq
         return self
 
@@ -723,6 +737,8 @@ class Archive(object):
         nsubint, npol, nchan, nbin = self.shape(squeeze=False)
         if DM is None:
             DM = self.getDM()
+        if DM is None: #for some ill-defined PSRFITS files
+            return
         cfreq = self.getCenterFrequency(weighted=wcfreq)
 
         K = 4.149e3
@@ -791,7 +807,7 @@ class Archive(object):
         return self.opw
 
     def calculateTemplate(self, mode='vonmises', sigma=None, lam=None,
-                          window_length=11, polyorder=5, **kwargs):
+                          window_length=11, polyorder=5, normalize=True, **kwargs):
         """
         Calculate a template shape
         """
@@ -806,9 +822,12 @@ class Archive(object):
         else:
             template = None
             self.template = None
+
         if template is not None:
             self.template = SP.SinglePulse(template, windowsize=int(self.getNbin()//8))
             self.opw = self.template.opw
+            self.template.normalize()
+            
         return self.template
 
     def superprep(self):
@@ -863,8 +882,10 @@ class Archive(object):
 
         self.data = np.roll(self.data, diff, axis=-1)
         self.average_profile = np.roll(self.average_profile, diff)
-        #print "diff", diff, diff*self.getTbin()
-        self.channel_delays += Decimal(str(-1*diff*self.getTbin())) #this is unnecessary? FIX THIS
+        tbin = self.getTbin()
+        if tbin is None: #error catching
+            return 
+        self.channel_delays += Decimal(str(-1*diff*tbin)) #this is unnecessary? FIX THIS
         self.calculateOffpulseWindow()
         return self
 
@@ -882,8 +903,7 @@ class Archive(object):
     def getLevels(self, differences=False):
         """ Returns calibration levels if this is a calibrator"""
         if not self.isCalibrator():
-            print("Not a calibration file")
-            return
+            raise TypeError("Not a calibration file")
         # Ensure data are scrunched in time,
         # or ignore this and simply calculate the weighted time average?
         self.tscrunch()
@@ -896,41 +916,43 @@ class Archive(object):
 
         # Check header info CAL_DCYC, CAL_NPHS, etc, to determine on-diode
         # or take an absolute value?
-        first = np.mean(data[0, :, :nbin/2])
-        second = np.mean(data[0, :, nbin/2:])
+        first = np.mean(data[0, :, :nbin//2])
+        second = np.mean(data[0, :, nbin//2:])
         if first > second:
-            highinds = np.arange(0, nbin/2)
-            lowinds = np.arange(nbin/2, nbin)
+            highinds = np.arange(0, nbin//2)
+            lowinds = np.arange(nbin//2, nbin)
         else:
-            lowinds = np.arange(0, nbin/2)
-            highinds = np.arange(nbin/2, nbin)
+            lowinds = np.arange(0, nbin//2)
+            highinds = np.arange(nbin//2, nbin)
 
         # Calculate calibrations
         freqs = self.getAxis('F')
         if differences:
-            caldata = np.zeros((npol, nchan))
-            calerrs = np.zeros((npol, nchan))
-            for i in xrange(npol):
-                for j in xrange(nchan):
-                    caldata[i, j] = np.mean(data[i, j, highinds]) - np.mean(data[i, j, lowinds])
-                    calerrs[i, j] = np.sqrt(np.std(data[i, j, highinds])**2 /
-                                            len(highinds) + np.std(data[i, j, lowinds])**2 / len(lowinds))
+            # Use numpy for speed-ups. Has dimensions of npol x nchan
+            caldata = np.mean(data[:, :, highinds], axis=-1) - np.mean(data[:, :, lowinds], axis=-1)
+            calerrs = np.sqrt(np.std(data[:, :, highinds], axis=-1)**2 / len(highinds) +
+                              np.std(data[:, :, lowinds], axis=-1)**2 / len(lowinds))
+
         else:
-            caldatalow = np.zeros((npol, nchan))
-            caldatahigh = np.zeros((npol, nchan))
-            calerrslow = np.zeros((npol, nchan))
-            calerrshigh = np.zeros((npol, nchan))
-            for i in xrange(npol):
-                for j in xrange(nchan):
-                    caldatalow[i, j] = np.mean(data[i, j, lowinds])
-                    caldatahigh[i, j] = np.mean(data[i, j, highinds])
-                    calerrslow[i, j] = np.std(data[i, j, lowinds]) / np.sqrt(len(lowinds))
-                    calerrshigh[i, j] = np.std(data[i, j, highinds]) / np.sqrt(len(highinds))
+            caldatalow = np.mean(data[:, :, lowinds], axis=-1)
+            caldatahigh = np.mean(data[:, :, highinds], axis=-1)
+            calerrslow = np.std(data[:, :, lowinds], axis=-1) / np.sqrt(len(lowinds))
+            calerrshigh = np.std(data[:, :, highinds], axis=-1) / np.sqrt(len(highinds))
+
 
         if differences:
             return freqs, caldata, calerrs
         return freqs, caldatalow, caldatahigh, calerrslow, calerrshigh
 
+
+    def getPulsarCalibrator(self):
+        """ Gets a Calibrator() object for a basic pulsar scan calibration """
+        freqs, caldata, calerrs = self.getLevels(differences=True)
+        pol_type = self.subintheader['POL_TYPE']
+        fd_poln = self.header['FD_POLN']
+        return Calibrator(freqs, caldata, calerrs, pol_type=pol_type, fd_poln=fd_poln, Funit=self.getFrequencyUnit(), Sunit=self.getDataUnit(), verbose=self.verbose)
+
+    
     def calibrate(self, psrcalar, fluxcalonar=None, fluxcaloffar=None):
         """Calibrates using another archive"""
         self.record(inspect.currentframe())
@@ -973,7 +995,7 @@ class Archive(object):
     @data.setter
     def data(self, value):
         self._data = value
-        self.weighted_data = value*self.weights[:,None,:,None]
+        self.weighted_data = value*self.weights[:,None,:,None]/np.sum(self.weights)
 
     def getData(self, squeeze=True, setnan=None, weight=True):
         """Returns the data array,  fully squeezed"""
@@ -1097,11 +1119,12 @@ class Archive(object):
 
     def getFrequencies(self, **kwargs):
         """Convenience function for getAxis"""
-        return getAxis('F', **kwargs)
-
+        return self.getAxis('F', **kwargs)
+    getFreqs = getFrequencies
+    
     def getTimes(self, **kwargs):
         """Convenience function for getAxis"""
-        return getAxis('T', **kwargs)
+        return self.getAxis('T', **kwargs)
 
     #Assumes the shape of data is (t, f, b) (i.e. polarization scrunched)
     def getPulse(self, t, f=None):
@@ -1177,7 +1200,8 @@ class Archive(object):
     #just bscrunch this?
     def getDynamicSpectrum(self, window=None, template=None, mpw=None,
                            align=None, windowsize=None, weight=True,
-                           verbose=False, snr=False, maketemplate=False):
+                           verbose=False, snr=False, maketemplate=False,
+                           debug=False):
         """
         Return the dynamic spectrum
         window: return the dynamic spectrum using only a certain phase bins
@@ -1204,6 +1228,8 @@ class Archive(object):
 
             Fedges = self.getAxis('F', edges=True)
             Tedges = self.getAxis('T', edges=True)
+            Funit = u.unitchanger(self.getFrequencyUnit())
+            Tunit = u.unitchanger(self.getTimeUnit())
 
             if template is not None:
                 if isinstance(template, SP.SinglePulse):
@@ -1225,7 +1251,7 @@ class Archive(object):
                     if windowsize >= len(sptemp.data):
                         raise IndexError("Improper window size")
                     sptemp = SP.SinglePulse(sptemp.data, windowsize=windowsize)
-
+                    
                 gs = np.zeros((fullshape[0], fullshape[2]))
                 offs = np.zeros((fullshape[0], fullshape[2]))
                 sig_gs = np.zeros((fullshape[0], fullshape[2]))
@@ -1259,94 +1285,177 @@ class Archive(object):
                             sp = SP.SinglePulse(data[i, j], opw=sptemp.opw, align=align)
                             baseline = sp.getOffpulseNoise(mean=True) #get mean value of offpulse
                             spfit = sp.fitPulse(sptemp.data)
-                            #if spfit is None:
-                            #    print i,j, np.shape(data)
-                            #    plt.plot(data[i,j])
-                            #    plt.show()
-                            #    raise SystemExit
+                            if spfit is None and debug:
+                                print(i,j, np.shape(data))
+                                plt.plot(data[i,j])
+                                plt.show()
+                                #raise SystemExit
                             if spfit is not None:
                                 gs[i, j] = spfit[ind] #bhat
                                 offs[i, j] = baseline
                                 sig_gs[i, j] = spfit[4]
                     gs, offs, sig_gs = wrapfunc(gs), wrapfunc(offs), wrapfunc(sig_gs)
                 #return wrapfunc(gs), wrapfunc(offs), wrapfunc(sig_gs)
-                return DS.DynamicSpectrum(gs, offs, sig_gs, F=Fedges, T=Tedges)
+                return DS.DynamicSpectrum(gs, offs, sig_gs, F=Fedges, T=Tedges, Funit=Funit, Tunit=Tunit)
 
             #kind of hard wired
             if window is None:
-                return DS.DynamicSpectrum(wrapfunc(np.mean(data, axis=2)), F=Fedges, T=Tedges)
+                return DS.DynamicSpectrum(wrapfunc(np.mean(data, axis=2)), F=Fedges, T=Tedges, Funit=Funit, Tunit=Tunit)
             else:
-                return DS.DynamicSpectrum(wrapfunc(np.mean(data[:, :, window], axis=2)), F=Fedges, T=Tedges)
+                return DS.DynamicSpectrum(wrapfunc(np.mean(data[:, :, window], axis=2)), F=Fedges, T=Tedges, Funit=Funit, Tunit=Tunit)
 
     def plot(self, ax=None, show=True):
         """Basic plotter of data"""
         data = self.getData()
         if len(np.shape(data)) == 1:
             if ax is None:
-                plt.plot(data, 'k')
-                plt.xlim(0, len(data))
+                fig = plt.figure()
+                ax = fig.add_subplot(111)
+            ax.plot(np.arange(len(data), dtype=np.float)/len(data), data, 'k')
+            ax.set_xlim(0, 1)
+            ax.set_xlabel("Pulse Phase")
+            unit = self.getDataUnit()
+            if unit == "Janksy":
+                ax.set_ylabel("Flux Density (%s)"%u.unitchanger(unit))
             else:
-                ax.plot(data, 'k')
-                ax.set_xlim(0, len(data))
+                ax.set_ylabel("Intensity")
+            #if "SCALE" in self.subintheader:
+            #    ax.set_ylabel(self.subintheader["SCALE"]) #this is still off
             if show:
                 plt.show()
         else:
-            print("Invalid dimensions")
+            raise IndexError("Invalid dimensions for plot()")
+            
     def imshow(self, ax=None, cbar=False, mask=None, show=True,
-               filename=None, setnan=0.0, **kwargs):
-        """Basic imshow of data"""
+               filename=None, setnan=0.0, cmap=None, **kwargs):
+        """ 
+        Basic imshow of data
+
+        Parameters
+        ----------
+        ax : matplotlib.axes._subplots.AxesSubplot
+            matplotlib Axes to draw on. If None, makes a new figure
+        cbar : bool
+            Draw a color bar
+        mask : float
+            If set to a value, use a masked array and set the mask 
+            to that value
+        show : bool
+            If true, plot the image
+        filename : str
+            Filename to save the figure to
+        setnan : float
+            Value to set NaNs to in the plot
+        cmap : str or matplotlib.colors.Colormap
+            Either a matplotlib colormap object or a string of the 
+            name of the colormap. Default is cividis if available,
+            otherwise viridis
+
+        Returns
+        -------
+        ax : matplotlib.axes._subplots.AxesSubplot
+            Returns the matplotlib Axes object.
+        """
         data = self.getData(setnan=setnan)
+        shape = self.shape(squeeze=False)
+
+        if ax is None:
+            fig = plt.figure()
+            ax = fig.add_subplot(111)
+        
         if len(np.shape(data)) == 2:
+            extent = None
+            if shape[0] == 1 and shape[1] == 1:
+                mode = "freq-phase"
+                Fedges = self.getAxis('F', edges=True) #is this true?
+                extent = [0, 1, Fedges[0], Fedges[-1]]
+            elif shape[1] == 1 and shape[2] == 1:
+                mode = "time-phase"
+                Tedges = self.getAxis('T', edges=True) #is this true?
+                extent = [0, 1, Tedges[0], Tedges[-1]]
+            elif shape[1] == 1 and shape[3] == 1:
+                mode = "freq-time"
+                Fedges = self.getAxis('F', edges=True)
+                Tedges = self.getAxis('T', edges=True)
+                extent = [Tedges[0], Tedges[-1], Fedges[0], Fedges[-1]]
+                data = np.transpose(data) #flip to freq on y axis
+            else:
+                raise IndexError("Unimplemented shape for imshow()")
+
+
+            #cmap.set_bad(color='k', alpha=1.0)
+                
             if mask is not None:
-                u.imshow(ma.masked_array(data, mask=mask), ax=ax, **kwargs)
+                u.imshow(ma.masked_array(data, mask=mask), ax=ax, extent=extent, cmap=cmap, **kwargs)
             else:
-                u.imshow(data, ax=ax, **kwargs)
-            if cbar:
-                plt.colorbar()
-            if filename is not None:
-                plt.savefig(filename)
-            if show:
-                plt.show()
-            else:
-                plt.close()
+                u.imshow(data, ax=ax, extent=extent, cmap=cmap, **kwargs)
+
+
+            if mode == "freq-phase":
+                ax.set_xlabel("Pulse Phase")
+                unit = u.unitchanger(self.getFrequencyUnit())
+                ax.set_ylabel("Frequency (%s)"%unit)
+                ax2 = ax.twinx()
+                ax2.set_ylim(0, self.getNchan())
+                ax2.set_ylabel("Channel Number")
+            elif mode == "time-phase":
+                ax.set_xlabel("Pulse Phase")
+                unit = u.unitchanger(self.getTimeUnit())
+                ax.set_ylabel("Time (%s)"%unit)
+                ax2 = ax.twinx()
+                ax2.set_ylim(0, self.getNsubint())
+                ax2.set_ylabel("Subintegration Number")
+            elif mode == "freq-time":
+                unit = u.unitchanger(self.getTimeUnit())
+                ax.set_xlabel("Time (%s)"%unit)
+                unit = u.unitchanger(self.getFrequencyUnit())
+                ax.set_ylabel("Frequency (%s)"%unit)
+                ax_freq = ax.twinx()
+                ax_freq.set_ylim(0, self.getNchan())
+                ax_freq.set_ylabel("Channel Number")
+                ax_time = ax.twiny()
+                ax_time.set_xlim(0, self.getNsubint())
+                ax_time.set_xlabel("Subintegration Number")
+                
+        elif len(np.shape(data)) == 1:
+            # Trust that this is a single subintegration in time?
+            Tedges = self.getAxis('T', edges=True) #is this true?
+            extent = [0, 1, Tedges[0], Tedges[-1]]       
+
+            u.imshow(data[np.newaxis, ...], ax=ax, extent=extent, cmap=cmap, **kwargs)
+
+            ax.set_xlabel("Pulse Phase")
+            unit = u.unitchanger(self.getTimeUnit())
+            ax.set_ylabel("Time (%s)"%unit)
+            ax2 = ax.twinx()
+            ax2.set_ylim(0, self.getNsubint())
+            ax2.set_ylabel("Subintegration Number")                
+                
         else:
             raise IndexError("Invalid dimensions for plotting")
+
+
+        if cbar:
+            plt.colorbar()
+        if filename is not None:
+            plt.savefig(filename)
+        if show:
+            plt.show()
+
         return ax
 
     def pavplot(self, ax=None, mode="GTpd", show=True, wcfreq=True):
-        """Produces a pav-like plot for comparison"""
-        data = self.getData(setnan=0.0)
-        if len(np.shape(data)) == 2:
-            shape = self.shape(squeeze=False)
-            if ax is None:
-                fig = plt.figure()
-                ax = fig.add_subplot(111)
-            cmap = plt.cm.afmhot
-            cmap.set_bad(color='k', alpha=1.0)
-            if shape[0] == 1 and shape[1] == 1: #fix this to match mode
-                Fedges = self.getAxis('F', edges=True) #is this true?
-                u.imshow(self.getData(), ax=ax, extent=[0, 1, Fedges[0], Fedges[-1]], cmap=cmap)
-                ax.set_xlabel("Pulse Phase")
-                ax.set_ylabel("Frequency (MHz)")
-                ax.set_title("%s %s\nFreq %0.3f MHz BW: %0.3f Length %0.3f S/N %0.3f"%(self.getName(), self.filename, self.getCenterFrequency(weighted=wcfreq), self.getBandwidth(), self.getDuration(), self.getSN()))#get the basename?
-                ax2 = ax.twinx()
-                ax2.set_ylim(0, self.getNchan())
-                ax2.set_ylabel("Index")
-                if show:
-                    plt.show()
-            if shape[2] == 1 and shape[1] == 1: #fix this to match mode
-                Tedges = self.getAxis('T', edges=True) #is this true?
-                u.imshow(self.getData(), ax=ax, extent=[0, 1, Tedges[0], Tedges[-1]], cmap=cmap)
-                ax.set_xlabel("Pulse Phase")
-                ax.set_ylabel("Time") #units
-                #ax.set_title("%s %s\nFreq %0.3f MHz BW: %0.3f Length %0.3f S/N %0.3f"%(self.getName(),self.filename,self.getCenterFrequency(weighted=wcfreq),self.getBandwidth(),self.getDuration(),self.getSN()))#get the basename?
-                ax2 = ax.twinx()
-                ax2.set_ylim(0, self.getNchan())
-                ax2.set_ylabel("Index")
-                if show:
-                    plt.show()
-        else:
-            print("Invalid dimensions")
+        """
+        Produces a pav-like plot for comparison
+        Built on top of imshow()
+        """
+        cmap = copy.copy(plt.cm.afmhot)
+        cmap.set_bad(color='k', alpha=1.0)
+        ax = self.imshow(ax=ax, show=False, cmap=cmap)
+        ax.set_title("%s %s\nFreq %0.3f MHz BW: %0.3f Length %0.3f S/N %0.3f"%(self.getName(), self.filename, self.getCenterFrequency(weighted=wcfreq), self.getBandwidth(), self.getDuration(), self.getSN()))#get the basename?
+
+        if show:
+            plt.show()
         return ax
 
     def joyDivision(self, border=0.1, labels=False, album=True, **kwargs):
@@ -1414,7 +1523,7 @@ class Archive(object):
             if show:
                 plt.show()
         else:
-            print("Invalid dimensions")
+            raise IndexError("Invalid dimensions for waterfall()")
 
     ### NOTE: THIS NEEDS TO BE CHECKED WITH THE NEW CHANGES ###
 
@@ -1587,13 +1696,14 @@ class Archive(object):
         """Returns period of the pulsar"""
         if self.isCalibrator():
             return 1.0/self.header['CAL_FREQ']
-        #if self.params is None:FOO
-        #    return None
-        if header or self.polyco is None:
+
+        if header or self.polyco is None and self.params is not None:
             return self.params.getPeriod()
         else:
-            P0 = self.polyco.calculatePeriod()
-            return P0
+            if self.polyco is not None:
+                P0 = self.polyco.calculatePeriod()
+                return P0
+            return
         
             #print P0,self.params.getPeriod()
             if np.abs(P0) < 1e-5: #Problem with large DT POLYCO values?
@@ -1635,7 +1745,15 @@ class Archive(object):
 
     def getTbin(self, numwrap=float):
         """Returns the time per bin"""
-        return numwrap(self.getPeriod()) / numwrap(self.getNbin())
+        P = self.getPeriod()
+        if P is None: #error catching
+            # try to grab out of HISTORY
+            dt = self.history.getLatest("TBIN")
+            if dt is None:
+                warnings.warn("Unable to obtain bin times")
+                return
+            return dt
+        return numwrap(P) / numwrap(self.getNbin())
 
     def getDM(self, numwrap=float):
         """Returns the data header DM"""
@@ -1696,7 +1814,37 @@ class Archive(object):
             return self.history.getLatest('CTR_FREQ')
         else:
             return self.header['OBSFREQ'] #perhaps do an unweighted version from DAT_FREQ?
+    getCenterFreq = getCenterFrequency
+        
+    def getFrequencyUnit(self):
+        for key in self.subintheader.keys():
+            if "TTYPE" in key:
+                if self.subintheader[key] == "DAT_FREQ":
+                    return self.subintheader[key.replace("TYPE", "UNIT")]
+        return None
+    getFreqUnit = getFrequencyUnit
 
+    def getTimeUnit(self):
+        if "INT_UNIT" in self.subintheader:
+            return self.subintheader["INT_UNIT"]
+        return None
+
+    def getDataUnit(self):
+        """ Returns the `intensity' unit of the data 
+        Should also try to look at the 
+        """
+        for key in self.subintheader.keys():
+            if "TTYPE" in key:
+                if self.subintheader[key] == "DATA":
+                    return self.subintheader[key.replace("TYPE", "UNIT")]
+        if "SCALE" in self.subintheader:
+            return self.subintheader["SCALE"]
+        return None
+    getScaleUnit = getDataUnit
+    getIntensityUnit = getDataUnit
+    getFluxDensityUnit = getDataUnit
+    getFluxUnit = getDataUnit
+        
     def getTelescope(self):
         """Returns the telescope name"""
         return self.header['TELESCOP']
@@ -1771,7 +1919,7 @@ class History(object):
             try:
                 return self.dictionary[field][-1][num]
             except IndexError:
-                print("Entry out of range")
+                warnings.warn("History getValue(): Entry out of range")
                 return None
     def getLatest(self, field):
         """Returns the latest key value"""
