@@ -4,6 +4,7 @@ Michael Lam 2015
 Useful mathematical commands
 '''
 import sys
+import copy
 import numpy as np
 import scipy.fftpack as fft
 import scipy.optimize as optimize
@@ -211,11 +212,25 @@ def decimate(x, window_len, error=False):#, mean=True, remainder=False):
             retval += x[i:length:window_len]
         return retval/window_len
 
-def imshow(x, ax=None, origin='lower', interpolation='nearest', aspect='auto', **kwargs):
+def imshow(x, ax=None, origin='lower', interpolation='nearest', aspect='auto', cmap=None, **kwargs):
+    # If cmap is a string, try to get the colormap,
+    # otherwise use default
+    if isinstance(cmap, str):
+        if cmap in plt.colormaps():
+            cmap = copy.copy(plt.get_cmap(cmap))
+        else:
+            cmap = None
+    # Try to use cividis as default colormap, otherwise viridis
+    #if cmap is None:
+    #    if "cividis" in plt.colormaps():
+    #        cmap = plt.cm.cividis
+    #    else:
+    #        cmap = plt.cm.viridis
+
     if ax is not None:
-        im = ax.imshow(x, origin=origin, interpolation=interpolation, aspect=aspect, **kwargs)
+        im = ax.imshow(x, origin=origin, interpolation=interpolation, aspect=aspect, cmap=cmap, **kwargs)
     else:
-        im = plt.imshow(x, origin=origin, interpolation=interpolation, aspect=aspect, **kwargs)
+        im = plt.imshow(x, origin=origin, interpolation=interpolation, aspect=aspect, cmap=cmap, **kwargs)
     return im
 
 '''
@@ -366,7 +381,7 @@ def write2Dtxt(filename, array, x=None, y=None, info=True, **kwargs):
 
 '''
 Normalize an array to unit height
-Below: normalize 
+Below: normalize
 '''
 def normalize(array, simple=False, minimum=None):
     if simple:
@@ -491,9 +506,9 @@ def fit_components(xdata, ydata, mode='gaussian', N=1, allownegative=False):
     else:
         imax = np.argmax(ydata)
     if mode == 'gaussian':
-        pinit = np.array([ydata[imax], xdata[imax], 0.02*nbins]) #2% duty cycle
+        pinit = np.array([ydata[imax], xdata[imax], 0.02]) #2% duty cycle
     elif mode == 'vonmises':
-        pinit = np.array([ydata[imax], xdata[imax], nbins])
+        pinit = np.array([ydata[imax], xdata[imax], 1.0/0.002])
     fitter = eval(mode)
 
     # perform this fit iteratively
@@ -501,6 +516,8 @@ def fit_components(xdata, ydata, mode='gaussian', N=1, allownegative=False):
         def fitfunc(p, x):
             retval = np.zeros(len(x))
             for i in range(n):
+                if not allownegative and p[3*i] <= 0:
+                    return 1e12 # large value
                 retval += fitter(x, p[3*i], p[3*i+1], p[3*i+2])
             return retval
         def errfunc(p, x, y):
@@ -518,9 +535,9 @@ def fit_components(xdata, ydata, mode='gaussian', N=1, allownegative=False):
         else:
             imax = np.argmax(resids)
         if mode == 'gaussian':
-            pinitprime = np.array([resids[imax], xdata[imax], 0.02*nbins]) #2% duty cycle
+            pinitprime = np.array([resids[imax], xdata[imax], 0.02]) #2% duty cycle
         elif mode == 'vonmises':
-            pinitprime = np.array([resids[imax], xdata[imax], nbins])#1.0/(0.02*nbins)])#need sqrt?
+            pinitprime = np.array([resids[imax], xdata[imax], 1.0/0.002])#1.0/(0.02*nbins)])#need sqrt?
         pinit = np.concatenate((pfit, pinitprime))
 
     s_sq = (errfunc(out[0], xdata, ydata)**2).sum()/(len(ydata)-len(pinit)-1) #-1 included here!
@@ -799,12 +816,19 @@ def shiftit_old(y, shift):
 def shiftit(y, shift):
     '''
     Speed-ups via Paul Baker, Ross Jennings
+
+    y : the array to shift
+    shift: the amount to shift
     '''
-    if isinstance(shift,np.ndarray):
-        shift = shift[...,np.newaxis]
-    N = y.shape[-1]
-    yfft = np.fft.rfft(y)
-    fs = np.fft.rfftfreq(N)#, d=dt)
+    if isinstance(shift, np.ndarray):
+        shift = shift[..., np.newaxis]
+    if y.__class__.__name__ == "SinglePulse" and y.yfft is not None:
+        yfft = y.yfft
+        fs = y.fs
+    else:
+        N = y.shape[-1]
+        yfft = np.fft.rfft(y)
+        fs = np.fft.rfftfreq(N)#, d=dt)
     phase = 1j*2*np.pi*fs*shift  #reversed from Paul's code so that this matches the previous convention
     yfft_sh = yfft * np.exp(phase)
     return np.fft.irfft(yfft_sh)
@@ -866,6 +890,115 @@ def tfresids(params, tfft, pfft):
     resids = np.abs(pfft[1:Nsum] - b*tfft[1:Nsum]*phasevec[1:Nsum])
     return resids
 
+
+
+def get_toa(template, profile, sigma_t, dphi_in=0.1, snrthresh=0., nlagsfit=5, norder=2):
+    """
+    Calculates TOA and its error in samples (bins).
+    Uses least-squares method in frequency domain, minimizing chi^2.
+    Also calculates scale factor for template matching.
+    Input: template = template file; if normalized to unity max,
+                      the scale factor divided by the input sigma_t is
+                      the peak to rms S/N.
+           profile = average profile to process
+           sigma_t = off pulse rms in same units as profile.
+
+    Output:
+    tauccf = TOA (bins) based on parabloic interpolation of CCF.
+    tauhat = TOA (bins) using Fourier-domain fitting.,
+    bhat = best-fit amplitude of pulse.
+    sigma_tau = error on tauhat.
+    sigma_b = error on bhat.
+    snr = bhat/sigma_t.
+    rho = cross correlation coefficient between template and centered profile.
+    """
+    classname = template.__class__.__name__
+
+
+    if classname == "SinglePulse" and template.yfft is not None:
+        istemplate = True
+        template_data = template.data
+        tfft = template.yfft #uses rfft, not tfft
+    elif classname == "SinglePulse":
+        istemplate = False
+        template_data = template.data
+        tfft = np.fft.fft(template_data)
+    else:
+        istemplate = False
+        template_data = template
+        tfft = np.fft.fft(template_data)
+
+
+    # Some initial values:
+    snr_coarse = np.max(profile)/sigma_t
+    tauhat = 0.
+    bhat = 0.
+    sigma_tau = -1.
+    sigma_b = -1.
+    rho = -2.
+
+    # find coarse estimates for scale factor and tau from CCF maximum
+    #  (quadratically interpolated)
+    ccf = np.correlate(template_data, profile, 'full')
+    lags = np.arange(-np.size(profile)+1., np.size(profile), 1.)
+    ccfmaxloc = ccf.argmax()
+    ccffit = ccf[ccfmaxloc-(nlagsfit-1)//2:ccfmaxloc+(nlagsfit-1)//2+1]
+    lagfit = lags[ccfmaxloc-(nlagsfit-1)//2:ccfmaxloc+(nlagsfit-1)//2+1]
+
+    p = np.polyfit(lagfit, ccffit, norder)
+    ccfhat = p[0] + p[1]*lagfit + p[2]*lagfit**2
+    tauccf = p[1]/(2.*p[2])
+
+    # roughly center the pulse to line up with the template:
+    ishift = int(-tauccf)
+    profile = np.roll(profile, ishift)
+
+    bccf = sum(template_data*profile)/sum(template_data**2)
+
+    # Search range for TOA using Fourier-domain method:
+    # expect -fwhm/2 < tauhat < fwhm/2  since pulse has been centered
+
+    # fwhm, taumin, taumax currently not used.  But should we do a
+    # windowed TOA calculation?
+    #fwhm = FWHM(template_data) # fwhm in samples (bins), switched to MTL code
+    #taumin = -fwhm/2.
+    #taumax = fwhm/2.
+
+
+    pfft = np.fft.fft(profile)
+    bhat0 = bccf
+    tauhat0 = tauccf+ishift
+    paramvec0 = np.array((bhat0, tauhat0))
+
+    paramvec = optimize.minpack.leastsq(tfresids, paramvec0, args=(tfft, pfft))
+    bhat = paramvec[0][0]
+    tauhat = paramvec[0][1]
+
+    sigma_tau, sigma_b = toa_errors_additive(tfft, bhat, sigma_t)
+    # snr = scale factor / sigma_t:
+    snr = (bhat*np.max(template_data))/sigma_t
+
+    if not istemplate:
+        # rho = correlation coefficient of template and shifted profile:
+        profile_shifted = shiftit(profile, +tauhat)	# checked sign: this is correct
+
+        # TBA: correct rho for off-pulse noise.
+        # Two possibilities:
+        #     1. subtract noise variance term  from sum(profile_shifted**2)
+        #  or 2. calculate ACF of profile_shifted with a one or two sample lag.
+        rho = np.sum(template_data*profile_shifted) / np.sqrt(np.sum(template_data**2)*np.sum(profile_shifted**2))
+    else:
+        # Reverse method
+        template_shifted = shiftit(template_data, -tauhat)
+        rho = np.sum(template_shifted*profile) / np.sqrt(np.sum(template_shifted**2)*np.sum(profile**2))
+
+    tauhat = tauhat - ishift	# account for initial shift
+    return tauccf, tauhat, bhat, sigma_tau, sigma_b, snr, rho
+
+
+
+
+
 def get_toa3(template, profile, sigma_t, dphi_in=0.1, snrthresh=0., nlagsfit=5, norder=2):
     """
     Calculates TOA and its error in samples (bins).
@@ -935,7 +1068,7 @@ def get_toa3(template, profile, sigma_t, dphi_in=0.1, snrthresh=0., nlagsfit=5,
     snr = (bhat*np.max(template))/sigma_t
 
     # rho = correlation coefficient of template and shifted profile:
-    profile_shifted = shiftit(profile, +tauhat)	# checked sign: this is correct
+    profile_shifted = shiftit(profile, +tauhat)	# checked sign: this is corr ct
 
     # TBA: correct rho for off-pulse noise.
     # Two possibilities:
@@ -944,3 +1077,23 @@ def get_toa3(template, profile, sigma_t, dphi_in=0.1, snrthresh=0., nlagsfit=5,
     rho = np.sum(template*profile_shifted) / np.sqrt(np.sum(template**2)*np.sum(profile_shifted**2))
     tauhat = tauhat - ishift	# account for initial shift
     return tauccf, tauhat, bhat, sigma_tau, sigma_b, snr, rho
+
+
+
+
+unitdict = {
+    "jansky" : "Jy",
+    "sec" : "s"
+}
+
+def unitchanger(unit):
+    """
+    If unit is in unitdict, convert
+    otherwise, return the old unit
+    """
+    if unit is None:
+        return None
+    lowerunit = unit.lower()
+    if lowerunit in unitdict:
+        return unitdict[lowerunit]
+    return unit
